#!/bin/sh

# httpd - This shell script starts/restart/stops the Apache Server

BASE=/fabos/webtools/bin
CONF=$BASE/httpd.conf
CTRL=$BASE/apachectl

RM=/bin/rm
SED=/bin/sed
MKDIR=/bin/mkdir
LOGDIR=/var/log/webtools
OUTDEVICE=$LOGDIR/webtools.alert

ARGC=$#
ARG=$1
INST_ID=$2
INST_IP=$3
INST_FCIP=$4
PIDDIR=/tmp/webtools
SOCKDIR=$PIDDIR/tmp_$INST_ID
LOGFILE=$PIDDIR/httpd.log
PLATFORM=

# Apache options

HTTPD=/usr/apache/bin/httpd
HTTPDOPTS="-f"

# Fabric OS env

FABOS_SWITCHNO=$INST_ID
SWITCHHOME=/fabos
export SWITCHHOME FABOS_SWITCHNO

logMessage() {
  msg=$*
	dt=`date '+%d%b%y_%H%M%S'`

	# Output to console always
  echo $msg

	# Output headers for first time
	if [ ! -f $OUTDEVICE ]; then
		echo "Webtools not available alert" >> $OUTDEVICE
		echo "----------------------------" >> $OUTDEVICE
		echo "Creation date: $dt" >> $OUTDEVICE
		echo "" >> $OUTDEVICE
		echo "Switch Instance Id -> $INST_ID" >> $OUTDEVICE
		echo "---------------------------" >> $OUTDEVICE
		echo "Reason: " >> $OUTDEVICE
		echo "------- " >> $OUTDEVICE
		echo "" >> $OUTDEVICE
	fi
	echo [$dt]: $msg >> $OUTDEVICE
}

vlog () {
  msg=$*
	dt=`date '+%d%b%y_%H%M%S'`
  if [ ! -d $PIDDIR ] ; then
    $MKDIR -p $PIDDIR
	  chmod 777 $PIDDIR
	fi
	if [ -f $LOGFILE ]; then
	  echo [$dt]: $msg >> $LOGFILE
	fi
}


checkPlatform()
{
  if [ ! -x /sbin/sin ] ; then
    logMessage "Unable to determine System Inventory."
    return 1 ;
  fi

	PLATFORM=`/sbin/sin | grep SWBD | cut -d ' ' -f 2 | cut -c 5-6 2>/dev/null`
	#if [ "$PLATFORM" != "10" -a "$PLATFORM" != "11" -a "$PLATFORM" != "12" -a "$PLATFORM" != "21" ]; then
	#	logMessage "Platform code $PLATFORM not supported."
	#	return 1 ;
  #fi
 	
	return 0;
}

# Validate Ip Address
# Argument - ip address in dot notation
# return 0 if valid, 1 otherwise

isValidIp()
{
   ip=$1

   if [ "$ip" = "0.0.0.0" ] || 
			[ "$ip" = "255.255.255.255" ] || [ "$ip" = "" ]; then
      return 1; 
	 fi

	 return 0;

}

# Validate both eth_ip and fc_ip address
# Argument order is eth_ip and fc_ip
# return 0 if valid, 1 otherwise

checkIpAddress()
{
   ethip=$1
	 fcip=$2

	 badEthIp=0
	 badFcIp=0

	 isValidIp $ethip

	 if [ $? -ne 0 ] ; then
		 badEthIp=1
   fi

	 isValidIp $fcip

	 if [ $? -ne 0 ] ; then
		 badFcIp=1
   fi

	 if [ "$badEthIp" = "1" -a "$badFcIp" = "1" ]; then
			logMessage "Invalid IP Address configuration for Switch[$INST_ID]."
      logMessage "Eth Ip: $ethip,  FcIp: $fcip"
			return 1
   fi


	 return 0;
}

checkPaths()
{
  # This directory must be created since it is not saved in the flash
	if [ -x $MKDIR ] ; then
		$MKDIR -p $LOGDIR ;
	  chmod 777 $LOGDIR
	else 
		# Assuming tmp must exist
		OUTDEVICE=/tmp/webtools.alert
		logMessage "Unable to create $LOGDIR directory, $MKDIR not found." ;
		exit 0 ;
	fi

	if [ ! -x $SED ] ; then
		logMessage "$SED not found, http server not started." ;
		exit 0 ;
	fi

	if [ ! -x $RM ] ; then
		logMessage "Warning: $RM not found" ;
	fi

  if [ ! -d $SOCKDIR ] ; then
    $MKDIR -p $SOCKDIR
	  chmod 777 $SOCKDIR
	fi

} 


# Set up configuration files for starting multiple instances

setupConfig()
{
	#echo Initializing apache for instance[$INST_ID] with BindAddress [$INST_IP] ...

	INST_CONF=`echo $CONF'.'$INST_ID`

	if [ $ARGC -ge 4 ] ; then
    isValidIp $INST_FCIP
    if [ $? -eq 0 ] ; then
	    FC_LISTEN=`echo Listen $INST_FCIP:80` 
    else
		  FC_LISTEN=""
    fi
	fi

	isValidIp $INST_IP
  if [ $? -eq 0 ] ; then
	  ETH_LISTEN=`echo Listen $INST_IP:80` 
  else
    ETH_LISTEN="$FC_LISTEN"
    FC_LISTEN=""
	fi

# /^\#Listen/s/\#Listen [.0-9.:]*/Listen '"$INST_IP:80"'/g
  $SED -e '
    /^\#Listen/s/\#Listen [.0-9.:]*/'"$ETH_LISTEN"'/g
		/^Listen/{G;s/$/'"$FC_LISTEN"'/;G;}
    /^ErrorLog/s/error_log/error_log_'"$INST_ID"'/g
    /^CustomLog/s/access_log/access_log_'"$INST_ID"'/g
    s/httpd.lock/httpd.lock.'"$INST_ID"'/g
    s/httpd.pid/httpd.pid.'"$INST_ID"'/g
    /FastCgiIpcDir/s|^.*|FastCgiIpcDir '"$SOCKDIR"'|g
   ' $CONF > $INST_CONF
#  $SED -e '
#		s/httpd.conf[.0-9.]*/httpd.conf.'"$INST_ID"'/g
#		s/httpd.pid[.0-9.]*/httpd.pid.'"$INST_ID"'/g
#		s/FABOS_SWITCHNO=[ 0-9 ]*/FABOS_SWITCHNO='"$INST_ID"'/g
#	 ' $CTRL > $CTRL.$INST_ID
#  chmod +x $CTRL.$INST_ID
}

cleanup()
{
	INST_CONF=`echo $CONF'.'$INST_ID`
	INST_CTRL=`echo $CTRL'.'$INST_ID`

	if [ -f $INST_CONF ] ; then
		vlog Removing file $INST_CONF ...
		$RM $INST_CONF ;
	fi

 	if [ -f $INST_CTRL ] ; then
		vlog Removing file $INST_CTRL ...
		$RM $INST_CTRL ;
	fi

	if [ -d $SOCKDIR ] ; then
		vlog Removing file $SOCKDIR ...
		$RM -rf $SOCKDIR ;
	fi

}

# This clean up is required if apache shutdown was not graceful
# Pid's stored in the files can be recycled for another process
# that will cause script to think httpd is running (via kill -0)

cleanupApacheFiles()
{
  INST_PID=`echo $PIDDIR/httpd.pid.$INST_ID`
  INST_LOCK=`echo $PIDDIR/httpd.lock.$INST_ID`

	# On Terminator type switch, there is no 
  # instance id

  PIDFILE=`echo $PIDDIR/httpd.pid`
	LOCKFILE=`echo $PIDDIR/httpd.lock`

	if [ -f $INST_PID ] ; then
		PID=`cat $INST_PID 2>/dev/null`
		if [ "x$PID" != "x" ] && kill -0 $PID 2>/dev/null ; then
			vlog "Process with pid [$PID] is still running ..."
		  vlog "File $INST_PID not removed." 
		else
		  vlog Removing file $INST_PID ...
		  $RM $INST_PID 2>/dev/null;
			if [ -f $INST_LOCK ] ; then
				vlog Removing file $INST_LOCK ...
				$RM $INST_LOCK 2>/dev/null;
			fi
			cleanup
		fi
	fi

	if [ -f $PIDFILE ] ; then
		PID=`cat $PIDFILE 2>/dev/null` 
		if [ "x$PID" != "x" ] && kill -0 $PID 2>/dev/null ; then
			vlog "Process with pid [$PID] is still running ..."
		  vlog "File $PIDFILE not removed." 
		else
		  vlog Removing file $PIDFILE ...
		  $RM $PIDFILE 2>/dev/null;
			if [ -f $LOCKFILE ] ; then
				vlog Removing file $LOCKFILE ...
				$RM $LOCKFILE 2>/dev/null;
			fi
			cleanup
		fi
	fi

	# remove after debugging

	if [ -f /core ] ; then
		vlog "httpd not started (crash) ?"
		$RM -f /core ;
	fi
}


# Check apache (http) status
# return 1, if still running, 0 otherwise

checkApacheStatus()
{
  INST_PID=`echo $PIDDIR/httpd.pid.$INST_ID`
  INST_LOCK=`echo $PIDDIR/httpd.lock.$INST_ID`

	# On Terminator type switch, there is no 
  # instance id

  PIDFILE=`echo $PIDDIR/httpd.pid`
	LOCKFILE=`echo $PIDDIR/httpd.lock`

	if [ -f $INST_PID ] ; then
		PID=`cat $INST_PID 2>/dev/null` 
		if [ "x$PID" != "x" ] && kill -0 $PID 2>/dev/null ; then
			vlog "Process with pid [$PID] is running ..."
			return 1;
		fi
	fi

	if [ -f $PIDFILE ] ; then
		PID=`cat $PIDFILE 2>/dev/null` 
		if [ "x$PID" != "x" ] && kill -0 $PID 2>/dev/null ; then
			vlog "Process with pid [$PID] is running ..."
			return 1;
		fi
	fi

	return 0;
}

# Start http server
# return 0 on success, 1 on failure

startApache()
{
	vlog "Platform id: $PLATFORM"

	case $PLATFORM in
		# Ulysses Switch
		10)
		  INST_CFG=$CONF.$INST_ID
		;;
		# Terminator Switch, Meteor, TermCR
		12|21|30)
		  INST_CFG=$CONF
		;;
		# Blazer/Dazzler[/IceTray) Switch CHECKME
		22|26|27|28|29)
		  INST_CFG=$CONF
		;;
		*) 
		  vlog "Unknown platform, httpd NOT started."
		;;
	esac	

	if [ ! -f $INST_CFG ]; then 
		logMessage "Missing configuration file: $INST_CFG"
		return 1
	fi

	if [ ! -x $HTTPD ]; then
		logMessage "Missing executable file: $HTTPD"
		return 1;
	fi

	# Launch Apache

	vlog "Starting http server [$INST_ID] ..."
	vlog $HTTPD $HTTPDOPTS $INST_CFG

	$HTTPD $HTTPDOPTS $INST_CFG

	sleep 2;
	checkApacheStatus

	if [ $? -ne 1 ] ; then
		vlog "httpd NOT started."
		cleanupApacheFiles
		return 1;
	else
		vlog "httpd started."
  fi

	return 0;
}

# Stop http server

stopApache()
{
  INST_PID=`echo $PIDDIR/httpd.pid.$INST_ID`
  INST_LOCK=`echo $PIDDIR/httpd.lock.$INST_ID`

	# On Terminator type switch, there is no 
  # instance id

  PIDFILE=`echo $PIDDIR/httpd.pid`
	LOCKFILE=`echo $PIDDIR/httpd.lock`

	if [ -f $INST_PID ] ; then
		PID=`cat $INST_PID 2>/dev/null` 
		if [ "x$PID" != "x" ] && kill -0 $PID 2>/dev/null ; then
			vlog "Stopping http server with pid [$PID] ..."
			kill -TERM $PID
		fi
	fi

	if [ -f $PIDFILE ] ; then
		PID=`cat $PIDFILE 2>/dev/null` 
		if [ "x$PID" != "x" ] && kill -0 $PID 2>/dev/null ; then
			vlog "Stopping http server with pid [$PID] ..."
			kill -TERM $PID
		fi
	fi

	return 0;
}

# Restart http server

restartApache()
{
  INST_PID=`echo $PIDDIR/httpd.pid.$INST_ID`
  INST_LOCK=`echo $PIDDIR/httpd.lock.$INST_ID`

	# On Terminator type switch, there is no 
  # instance id

  PIDFILE=`echo $PIDDIR/httpd.pid`
	LOCKFILE=`echo $PIDDIR/httpd.lock`

	if [ -f $INST_PID ] ; then
		PID=`cat $INST_PID 2>/dev/null` 
		if [ "x$PID" != "x" ] && kill -0 $PID 2>/dev/null ; then
			vlog "Sending HUP to http server with pid [$PID] ..."
			if kill -HUP $PID ; then
			  vlog "http server restarted."
			fi
		fi
	fi

	if [ -f $PIDFILE ] ; then
		PID=`cat $PIDFILE 2>/dev/null` 
		if [ "x$PID" != "x" ] && kill -0 $PID 2>/dev/null ; then
			vlog "Sending HUP to http server with pid [$PID] ..."
			if kill -HUP $PID ; then
			  vlog "http server restarted."
			fi
		fi
	fi

	return 0;
}

main()
{
	# bail out quietly if httpd does not exist

  if [ ! -f $HTTPD ] ; then 
		logMessage "Missing executable file: $HTTPD"
		exit 1;
	fi

	checkPaths

	checkPlatform
	if [ $? -ne 0 ] ; then
    exit 1;
  fi

	if [ $ARGC -ge 3 ] ; then
	  checkIpAddress $INST_IP $INST_FCIP
	  if [ $? -ne 0 ] ; then
        exit 1
    fi
	fi

	case $ARG in

		start)  
			 # start server
	     if [ $ARGC -ge 3 ] ; then
		      setupConfig
	     fi
			 checkApacheStatus
			 if [ $? -ne 1 ] ; then
			   cleanupApacheFiles
			   startApache
			 else
			   vlog "httpd is already running."
			 fi
			 ;; 

		stop)
			# stop server
			checkApacheStatus
			if [ $? -eq 1 ] ; then
				stopApache
				cleanupApacheFiles
				checkApacheStatus
				if [ $? -eq 0 ] ; then
			 	 vlog "httpd stopped."
				fi
			else	
			  vlog "httpd is NOT running."
			fi
			;;

		restart)
			# restart server
			checkApacheStatus
			if [ $? -eq 1 ] ; then
			  restartApache
			else
	      if [ $ARGC -ge 3 ] ; then
		       setupConfig
	      fi
				vlog "No http server, HUP will not be sent"
				startApache
			fi
			;;

		*)
			# usage
			echo "usage: $0 <start|stop|restart> InstanceNumber EthIpAddr [FcIpAddr]"
			;;
	esac
}


# Main function

main
