#!/bin/sh
#
#
#    Copyright (c) 1996-2000 Brocade Communications Systems, Inc.
#    All rights reserved.
#
#
# NAME
#      portpost2.sh - POST phase II support script
# 
# AVAILABILITY
#      internal
# 
# DESCRIPTION
#      This  is  not  a  command.  It  is a script that manages the
#      POST(power on self test) phase II operations, which involves
#      mainly  in functional tests. The script is designed to oper-
#      ate across power cycles.
# 
# SEE ALSO
#      portpost1(1d)
# 
# CAUTIONS:
#      DO NOT use lb_mode 5 (internal loopback during POST for Bloom
#      based products. There is a defect in the Bloom that could
#      disrupt copper media when using lb_mode 5.
#

#  Init variables
#
nPost=0			# number of POST executed
exit_status=0		# this script's exit status value
slot=0			# target slot to run POST
slot_message=""         # message related to slot information

#  special temp file used for each test result
#
logFile=/tmp/post2Log

#
#  function to process trapped signal received. Upon entry, KILL the
#  child PID and output the abort message.
#
post_cleanup()
{
	if [ $?child ]
	then
		/bin/kill $child
	fi
	#
	#  display ALL performed test result
	#
	_current_time=`/bin/date \+\%s`
	_date=`/bin/date`
	_duration=`/usr/bin/expr $_current_time - $startTime`
	_minute=`/usr/bin/expr $_duration / 60`
	_hour=`/usr/bin/expr $_minute / 60`
	_sec=`/usr/bin/expr $_duration % 60`
	_minute=`/usr/bin/expr $_minute % 60`
	/bin/echo "" >> $logFile.$slot
	/bin/echo "POST2:$slot_message POST2 script has been ABORTED $_date took ($_hour:$_minute:$_sec)"
	/bin/echo "POST2:$slot_message POST2 script has been ABORTED $_date took ($_hour:$_minute:$_sec)" >> $logFile.$slot
	/bin/echo "POST2:$slot_message Completed $nPost Diagnostic tests."
	/bin/echo "POST2:$slot_message Completed $nPost Diagnostic tests." >> $logFile.$slot
	exit 1
}


#
#  Function to wait for exec_diag return and process the return status.
#
checkStatus()
{
	#  wait for current child to finish and get return status
	#
	export child=$!
	wait $child
	rval=$?
	
	#  construct PASSED or FAILED msg from return value
	#  .  return value = 0 for PASSED
	#  .  return value != 0 for FAILED
	#
	case $rval in
	139 | 237 )
		echo " SKIPPED $1 due to Known Issue ($rval). Please run following command manually."
		echo "$*"
		echo " SKIPPED $1 due to Known Issue ($rval). Please run following command manually." >>  $logFile.$slot
		echo "$*" >>  $logFile.$slot
		;;
	0 )
		status=PASSED;;
	* )
		status=FAILED
		exit_status=`/usr/bin/expr $exit_status + 1`;;
	esac

	#  compute return status (positive or negative value)
	#
	if [ $rval -gt 127 ]
	then
		rval=`/usr/bin/expr $rval - 256`
	fi

	/bin/echo "POST2:$slot_message Returned status = $rval ($status)" >> $logFile.$slot
}

if [ $?FABOS_SLOTNO ]
then
	slot=$FABOS_SLOTNO
fi

#
# source diagcommon
#
home="/fabos/share"
util="diagcommon.sh"
ok=0
for f in "./$util" "$FABOSHOME/share/$util" "$home/$util" ; do
	if [ -r $f ] ; then
		. $f
		ok=1
		break;
	fi
done
if [ $ok -ne 1 ] ; then
	err "Could not locate $util"
	exit 3
fi

#
# get sys info for multi blade
#
multiBlade=`isMultiBlade`
if [ $multiBlade = TRUE ]
then
     slot_message="Slot $slot"
     export syntax="$0 -slot [slot] essScript"
else
     export syntax="$0 essScript"
fi

# cleanup the old log file
if [ -f $logFile.$slot ]
then
	/bin/rm $logFile.$slot
fi

#
#  display starting message
#
startTime=`/bin/date \+\%s`
_date=`/bin/date`
/bin/echo "POST2:$slot_message Started running $_date"
/bin/echo "POST2:$slot_message Started running $_date" >> $logFile.$slot


#  export environment variable
#
DIAG_POST=0 ; export DIAG_POST

#
# Find exec_diag
#
cmd=`commandPath exec_diag`

#
# Get list of ports for those diags that need them
#
user_ports=`$cmd diagshow -use_bports 1 | /bin/grep "   OK" | /usr/bin/cut -d " " -f 2 | /bin/grep -v "\-\-" | /bin/sort -n | /usr/bin/xargs /bin/echo`

# Make sure there is a port to run
#
if [ -z "$user_ports" ] ;
then
     /bin/echo "POST2:$slot_message ABORT - Nothing to test (NO active ports)"
     /bin/echo "POST2:$slot_message ABORT - Nothing to test (NO active ports)" >> $logFile.$slot
     exit 3 ;
else
     /bin/echo "POST2:$slot_message Testing User ports: $user_ports" >> $logFile.$slot
fi


#  export environment variable
#
DIAG_POST=1 ; export DIAG_POST
DCOMM_USR_FAILLIM=1 ; export DCOMM_USR_FAILLIM
setdbg DIAG 1

#  setup signal trap to call cleanup() if the followings:
#	control/d entered at the same window
#	kill <pid> is entered
#
trap "" 2
trap post_cleanup INT TERM KILL

#  reset Error on each port 
# 
/bin/echo "POST2:$slot_message Running diagclearerror" >> $logFile.$slot
$cmd diagclearerror $slot 2>&1 >> $logFile.$slot

# detect and handle bplb_mode
bplb_mode=`getConfig diag.mode.bplb 2 0`
if [ "$bplb_mode" != "0" ]
then
  echo "POST2:$slot_message BPLB mode is ON"
  echo "POST2:$slot_message BPLB mode is ON" >> $logFile.$slot
fi

#
# Start testing
#
#  perform CAM test
#
nPost=`/usr/bin/expr $nPost + 1`
/bin/echo "" >> $logFile.$slot
/bin/echo "POST2:$slot_message Test #$nPost - Running camtest" >> $logFile.$slot
/bin/echo "POST2:$slot_message Test #$nPost - Running camtest"
$cmd camtest 2>&1 >> $logFile.$slot &
checkStatus camtest

#  perform txdpath test
#
nPost=`/usr/bin/expr $nPost + 1`
/bin/echo "" >> $logFile.$slot
/bin/echo "POST2:$slot_message Test #$nPost - Running txdpath" >> $logFile.$slot
/bin/echo "POST2:$slot_message Test #$nPost - Running txdpath"
$cmd txdpath -nframes 5 -lb_mode 2 >> $logFile.$slot 2>&1 >> $logFile.$slot&
checkStatus txdpath -nframes 5 -lb_mode 2

#  perform Minicycle (serdes) test
#
nPost=`/usr/bin/expr $nPost + 1`
/bin/echo "" >> $logFile.$slot
/bin/echo "POST2:$slot_message Test #$nPost - Running minicycle (SERDES)" >> $logFile.$slot
/bin/echo "POST2:$slot_message Test #$nPost - Running minicycle (SERDES)"
$cmd minicycle -lb_mode 2 -nmegs 1 2>&1 >> $logFile.$slot &
checkStatus minicycle -lb_mode 2 -nmegs 1

#  perform backplane connection test
#  (Note that this test MUST be called as last test in POST)
#
nPost=`/usr/bin/expr $nPost + 1`
    /bin/echo "" >> $logFile.$slot
if [ "$bplb_mode" = "" -o "$bplb_mode" = "0" ]
then
    /bin/echo "POST2:$slot_message Test #$nPost - Running minicycle splb_mode backplane" >> $logFile.$slot
    /bin/echo "POST2:$slot_message Test #$nPost - Running minicycle splb_mode backplane"
    $cmd minicycle -lb_mode 8 -nmegs 1 -splb_mode 1 2>&1 >> $logFile.$slot &
    checkStatus minicycle -lb_mode 8 -nmegs 1 -splb_mode 1
else
    /bin/echo "POST2:$slot_message Test #$nPost - Skipping minicycle splb_mode backplane(BPLB mode ON)" >> $logFile.$slot
    /bin/echo "POST2:$slot_message Test #$nPost - Skipping minicycle splb_mode backplane(BPLB mode ON)"
fi
#  check test status of each port (must be RUN in fore ground)
# 
/bin/echo "POST2:$slot_message Running diagshow" >> $logFile.$slot
/bin/echo "POST2:$slot_message Running diagshow"
$cmd diagshow --slot $slot -use_bports 1 -silent 1 >> $logFile.$slot 
status=$?

if [ $status -ne 0 ]
then
    exit_status=`/usr/bin/expr $exit_status + 1`
fi
#  display ALL performed test result
#
/bin/echo "" >> $logFile.$slot
/bin/echo "POST2:  ********** $slot_message POST Summary  **********" >> $logFile.$slot
/bin/echo "POST2:$slot_message Completed $nPost Diagnostic test:" >> $logFile.$slot

#  display exit msg and exit script
#
if [ $exit_status -ne 0 ]
then
    status=FAILED
else
    status=PASSED
fi
_current_time=`/bin/date \+\%s`
_date=`/bin/date`
_duration=`/usr/bin/expr $_current_time - $startTime`
_minute=`/usr/bin/expr $_duration / 60`
_hour=`/usr/bin/expr $_minute / 60`
_sec=`/usr/bin/expr $_duration % 60`
_minute=`/usr/bin/expr $_minute % 60`
/bin/echo "POST2:$slot_message Script $status with exit status of $exit_status $_date took ($_hour:$_minute:$_sec)"
/bin/echo "POST2:$slot_message Script $status with exit status of $exit_status $_date took ($_hour:$_minute:$_sec)" >> $logFile.$slot
exit $exit_status
